// Global variables
var codes = {
    "113":"00", "116":"01", "119":"02", "122":"03", "143":"04", "176":"05", "179":"06", "182":"07",
    "185":"08", "200":"09", "227":"10", "230":"11", "248":"12", "260":"13", "263":"14", "266":"15",
    "281":"16", "284":"17", "293":"18", "296":"19", "299":"20", "302":"21", "305":"22", "308":"23",
    "311":"24", "314":"25", "317":"26", "320":"27", "323":"28", "326":"29", "329":"30", "332":"31",
    "335":"32", "338":"33", "350":"34", "353":"35", "356":"36", "359":"37", "362":"38", "365":"39",
    "368":"40", "371":"41", "374":"42", "377":"43", "386":"44", "389":"45", "392":"46", "395":"47"
};
var smsRequest, smsContent, points, error, forecastTimes, SMSnumber,
    offset = [],
    latitude = [],
    longitude = [],
    weatherOL = [],
    hourCount = [];

// Called when application is started.
function OnStart() {
	//Create layout that fills the screen.
	lay = app.CreateLayout( "Linear", "FillXY,VCenter" );
	edt = app.CreateTextEdit( "CB1", 0.8 ); 

	//Create a text control to show data log.
	txt = app.CreateText("", 0.9, 0.9, "Multiline, Left");
	txt.SetTextColor( "#ffffffff" );
	lay.AddChild(txt);

	//Add main layout to app.
	app.AddLayout(lay);
	
	//Create array to hold log messages.
	log = new Array(); 
	Log("Waiting for SMS...\n");

	// Create SMS object and set callbacks.
	sms = app.CreateSMS();
	sms.SetOnMessage(sms_OnMessage);
	
	// for testing, to be removed!
	//manageRequest();
}

function manageSMS() {
    var totalLength = smsContent.length;
    if(totalLength > 160) {
        var numberOfSMS = parseInt(totalLength / 160) + 1;
        totalLength += numberOfSMS * 3; // add 3 chars for each SMS 1/3, 2/3, 3/3...
        numberOfSMS = parseInt(totalLength / 160) + 1;
        if(numberOfSMS * 160 < totalLength) {
            ++numberOfSMS;
        }
        var count, offset = 0;
        Log("numberOfSMS: " + numberOfSMS + ", totalLength: " + totalLength);
        for (count = 1; count <= numberOfSMS; ++count) {
            Log(offset);
            SendSMS(count + "/" + numberOfSMS + smsContent.substr(offset, 157));
            offset += 157;
        }
    } else {
        SendSMS(smsContent);
    }
}

function SendSMS(SMSText) {
    Log(SMSText);
    // Use my number just for testing
    //sms.Send("0486057395", SMSText);
    sms.Send(SMSnumber, SMSText);
}

function latLong(lat, long) {
    return formatNumber(true, lat * 1000, 5) + formatNumber(true, long * 1000, 6);
}

//Extract times and locations from the SMS request
function decodeRequest() {
    var param = smsRequest.split("?");
    if (param[0] == "FORECAST-REQUEST") { // Ignorer le message si ce n'est pas une requête météo
        forecastTimes = param[1];
        var time = param[1].split("/");
        var locations = param[2].split("/");
        points = locations.length;
        if (points == 0) {
            smsContent = "Pas de location géoraphique !";
            error = true;
            return;
        }
        var count; //, coord = [];
        error = false;
        // Check if lat and long are valid
        for (count = 0; count < points; ++count) {
            //coord = locations[count].split(" "); // -18854+050537
            latitude[count] = parseInt(locations[count].substr(0, 6)) / 1000; // parseInt(coord[0]) / 1000;
            longitude[count] = parseInt(locations[count].substr(6, 7)) / 1000; // parseInt(coord[1])/ 1000;
            if (latitude[count] < -90.0 || latitude[count] > 90.0 || longitude[count] < -180.0 || longitude[count] > 180.0) {
                error = true;
                smsContent = "Une ou plusieurs des coordonnées géographiques sont invalides.\n";
                break;
            }
        }
        // Check if forecast times are valid
        if (time.length == 0) {
            smsContent = "Aucune heure de prévision !";
            error = true;
            return;
        }
        for (count = 0; count < time.length; ++count) {
            if (parseInt(time[count]) < 0 || parseInt(time[count]) > 99) {
                smsContent += "Les valeurs de temps de prévision météo sont invalides."
                error = true;
                break;
            }
        }
    } else {
        error = true;
    }
}

// Check incoming message, get the requested info and send it back to the requester
function manageRequest() {
    var point, params;
    // For testing, to be removed!
    //smsRequest = "FORECAST-REQUEST?06/12/18/24?-19069+050569/-19285+050540/-18854+050537";
    
    //Log(smsRequest);
	smsContent = "";
	error = false;
    decodeRequest();
	if (!error) {
        Log("Extracting information...");
    	var url = 'http://api.worldweatheronline.com';
	    var path = "/premium/v1/marine.ashx?key=7cc8ff94db004dde8c8153604181412&";
        for (point = 0; point < points; ++point) {
            //done[point] = false;
            weatherOL[point] = "";
	        params = "format=xml&tide=yes&q=" + latitude[point] + "," + longitude[point];
            SendRequest(point, url + path + params);
	        smsContent += latLong(latitude[point], longitude[point]);
            ExtractWeatherInfo(point);
        }
	}
	//Log(smsContent);
	//Log(smsContent.length);
    //var file = "/sdcard/sms.txt";
    //app.WriteFile(file, smsContent);
    if (smsContent != "") {
	    manageSMS();
    }
}

//Show incomming SMS messages.
function sms_OnMessage(number, msg) {
    smsRequest = msg;
	Log("SMS received from " + number + ": " + msg);
	SMSnumber = number;
	manageRequest();
}

// Send an http get request. 
function SendRequest(point, url) { 
	var httpRequest = new XMLHttpRequest();
	//Log ("Request from " + url);
	httpRequest.onreadystatechange = function() { HandleReply(point, httpRequest); };   
	app.ShowProgress("Loading..."); 
	httpRequest.open("GET", url, false); 
	httpRequest.send(null); 
	app.HideProgress(); 
} 

// Handle the server's reply. 
function HandleReply(point, httpRequest) { 
	//Log("point: " + point + ", readyState: " + httpRequest.readyState + ", status:" + httpRequest.status);
	if(httpRequest.readyState >= 3 && httpRequest.status == 200) { // If we got a valid response.
        weatherOL[point] = httpRequest.responseText;
        //Log(point + ": " + httpRequest.responseText.substr(41, 24));
        //done[point] = true;
	} //else { // An error occurred
	//	Log("Error!");
	//}
} 

//Add messages to log.
function Log(msg) {
	if (txt.GetLineTop(txt.GetLineCount()) >= 0.8) log.shift();
	log.push(msg + "\n");
	txt.SetText(log.join(""));
}

// Extract all weather info stored in weatherOL[point]
function ExtractWeatherInfo(point) {
    //Log("function ExtractWeatherInfo(" + point + ")");
    var pos, hour, day;
    hourCount[point] = 0;
    offset[point] = 0;
    for (day = 0; day < 4; ++day) { // 24, 48, 72, 96 ...
        //console.log ("offset[" + point + "] = " + offset[point]);
        pos = weatherOL[point].indexOf("<weather>", offset[point]);
        //if (pos == -1) break;
	    //smsContent += latLong(latitude[point], longitude[point]);
        GetDayValues(point);
        for (hour = 0; hour < 5; hour++) { // 0, 3, 6, 9 ...
            //console.log ("hourCount[" + point + "] = " + hourCount[point]);
            GetHourlyValues(point);
            if (hourCount[point] > 96) { break; }
        }
        if (hourCount[point] > 96) { break; }
    }
}

// Get the day values
function GetDayValues(point) {
    //Log("function GetDayValues(" + point + ")");
    var dateStr = shorterDate(GetKeyValue(point, "date"));
    //Log(dateStr);
    var sunriseStr = to24(GetKeyValue(point, "sunrise"));
    var sunsetStr = to24(GetKeyValue(point, "sunset"));
    var moonriseStr = to24(GetKeyValue(point, "moonrise"));
    var moonsetStr = to24(GetKeyValue(point, "moonset"));
    var moon_phaseStr = GetKeyValue(point, "moon_phase");
    var moon_illuminationStr = GetKeyValue(point, "moon_illumination");
    var maxtempCStr = GetKeyValue(point, "maxtempC");
    var mintempCStr = GetKeyValue(point, "mintempC");
    var tideTimeStr = GetKeyValue(point, "tideTime");
    var tideHeight_mtStr = GetKeyValue(point, "tideHeight_mt");
    var tide_typeStr = GetKeyValue(point, "tide_type");
}

Number.prototype.pad = function(size) {
  var s = String(this);
  while (s.length < (size || 2)) { s = "0" + s; }
  return s;
}

function formatNumber(leadingSign, number, length) {
    var absValue = Math.abs(number);
    var retStr;
    if (leadingSign) {
        if (number >= 0) { return "+" + absValue.pad(length); }
        else { return "-" + absValue.pad(length); }
    } else {
        return absValue.pad(length);
    }
}

// Get the hourly values
function GetHourlyValues(point) {
    //Log("function GetHourlyValues(" + point + ")");
    var timeStr = hourCount[point].pad(2); // toTime(GetKeyValue("time"));
    var tempCStr = formatNumber(true, parseInt(GetKeyValue(point, "tempC")), 2);
    var windspeedMilesStr = formatNumber(false, GetKeyValue(point, "windspeedMiles"), 2);
    var winddirDegreeStr = formatNumber(false, GetKeyValue(point, "winddirDegree"), 3);
    var weatherCodeStr = GetKeyValue(point, "weatherCode");
    var precipMMStr = GetKeyValue(point, "precipMM");
    var humidityStr = GetKeyValue(point, "humidity");
    var visibilityStr = formatNumber(false, GetKeyValue(point, "visibility"), 2);
    var pressureStr = formatNumber(false, GetKeyValue(point, "pressure"), 4);
    var cloudcoverStr = parseInt(parseFloat(GetKeyValue(point, "cloudcover")) * 0.08 + 0.5);
    var HeatIndexCStr = GetKeyValue(point, "HeatIndexC");
    var DewPointCStr = GetKeyValue(point, "DewPointC");
    var WindChillCStr = GetKeyValue(point, "WindChillC");
    var WindGustMilesStr = formatNumber(false, parseInt(GetKeyValue(point, "WindGustMiles")), 2);
    var FeelsLikeCStr = GetKeyValue(point, "FeelsLikeC");
    var sigHeight_mStr = formatNumber(false, parseInt(GetKeyValue(point, "sigHeight_m")) * 10, 3);
    var swellHeight_mStr = GetKeyValue(point, "swellHeight_m");
    var swellDirStr = formatNumber(false, GetKeyValue(point, "swellDir"), 3);
    var swellPeriod_secsStr = GetKeyValue(point, "swellPeriod_secs");
    var waterTemp_CStr = GetKeyValue(point, "waterTemp_C");
    if (forecastTimes.indexOf(timeStr) != -1) {
        //Log(forecastTimes + ", " + timeStr);
        smsContent += timeStr; // 2 (TT)
        smsContent += tempCStr; // 3 (±AA)
        smsContent += winddirDegreeStr; // 3 (WDI)
        smsContent += windspeedMilesStr; // 2 (WS)
        smsContent += WindGustMilesStr; // 2 (WG)
        smsContent += pressureStr; // 4 (PPPP)
        smsContent += visibilityStr; // 2 (VV)
        smsContent += cloudcoverStr; // 1 (C)
        smsContent += sigHeight_mStr; // 2 (SHM)
        smsContent += swellDirStr; // 3 (SDD)
        smsContent += codes[weatherCodeStr]; // 2 (CO) weatherCodeStr
        //console.log(timeStr + tempCStr + winddirDegreeStr + windspeedMilesStr + WindGustMilesStr + pressureStr + visibilityStr + cloudcoverStr + sigHeight_mStr + swellDirStr + codes[weatherCodeStr]);
        //console.log("TT±AAWDIWSWGPPPPVVCSHMSDDCO");
    }
    hourCount[point] += 3;
}

// Extract value between XML keyword (<keyword>Value</keyword>)
function GetKeyValue(point, keyword) {
    //console.log("GetKeyValue(" + point + ", " + keyword + ")");
    var startSTR = "<" + keyword + ">";
    var endSTR = "</" + keyword + ">";
    var len = startSTR.length;
    var pos1 = weatherOL[point].indexOf(startSTR, offset[point]) + len;
    var pos2 = weatherOL[point].indexOf(endSTR, offset[point]);
    offset[point] = pos2 + len + 1;
    return weatherOL[point].substr(pos1, pos2 - pos1);
}

// Convert to a formated time (0 -> 00, 0900 -> 09, 1500 -> 15)
function toTime(time) {
    if (time == "0") {
        return "00";
    } else {
        if (time.length == 3) {
            return "0" + time.substr(0, 1);
        } else {
            return time.substr(0, 2);
        }
    }
}

// Convert AM/PM time into 24h time
function to24(time) {
    var timeAMPM = time.split(' ');
    if (timeAMPM[1] != "AM") {
        return timeAMPM[0];
    } else {
        var hourMin = timeAMPM[0].split(':');
        return (parseInt(hourMin[0]) + 12) + ":" + hourMin[1];
    }
}

// Convert YYYY-MM-DD to YYYYMMDD
function shorterDate(date) {
    var ymd = date.split('-');
    return ymd[0] + ymd[1] + ymd[2];
}
